@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * bf_lib.h - non-GUI general functions
  *
- * Copyright (C) 2000 Olivier Sessink
+ * Copyright (C) 2000-2004 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #ifndef __BF_LIB_H_
@@ -58,13 +58,12 @@ void free_convert_table(Tconvert_table *tct);
 void utf8_offset_cache_reset();
 guint utf8_byteoffset_to_charsoffset_cached(gchar *string, glong byteoffset);
 
-gboolean change_dir(gchar * filename);
 gchar *strip_any_whitespace(gchar *string);
 gchar *trunc_on_char(gchar * string, gchar which_char);
 gchar *strip_common_path(char *to_filename, char *from_filename);
 gchar *most_efficient_filename(gchar *filename);
 gchar *create_relative_link_to(gchar * current_filepath, gchar * link_to_filepath);
-gchar *create_full_path(gchar * filename, gchar *basedir);
+gchar *create_full_path(const gchar * filename, const gchar *basedir);
 gchar *ending_slash(const gchar *dirname);
 gchar *path_get_dirname_with_ending_slash(const gchar *filename);
 gboolean file_exists_and_readable(const gchar * filename);
@@ -76,4 +75,5 @@ gchar *create_secure_dir_return_filename();
 void remove_secure_dir_and_filename(gchar *filename);
 /*gchar *buf_replace_char(gchar *buf, gint len, gchar srcchar, gchar destchar);*/
 void wordcount(gchar *text, guint *chars, guint *lines, guint *words);
+GList *glist_from_gslist(GSList *src);
 #endif /* __BF_LIB_H_ */
