@@ -15,9 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
-/* #define DEBUG*/
+
+/*#define DEBUG*/
 
 /*
  * This module deals with the GUI dialog used in bluefish to manage
@@ -33,6 +34,7 @@
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <string.h>  	/* strspn() */
+#include <strings.h>  	/* strcasecmp() */
 #include "bluefish.h"
 #include "bf_lib.h"
 
@@ -40,7 +42,7 @@
 #include "stringlist.h"
 #include "gtk_easy.h"
 
-#define STRING_MAX_SIZE 1024
+#define STRING_MAX_SIZE 4096
 #define MAX_ARRAY_LENGTH 20
 
 
@@ -362,17 +364,17 @@ void estrl_dialog(GList **which_list, gchar *title, gint what_list
 	}
 	gtk_clist_thaw(GTK_CLIST(estrl->clist));
 
-	but = bf_stock_button(_(" Add "), G_CALLBACK(estrl_add_clicked_lcb), estrl);
+	but = bf_generic_mnemonic_button(_(" Add "), G_CALLBACK(estrl_add_clicked_lcb), estrl);
 	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-	but = bf_stock_button(_(" Update "), G_CALLBACK(estrl_update_clicked_lcb), estrl);
+	but = bf_generic_mnemonic_button(_(" Update "), G_CALLBACK(estrl_update_clicked_lcb), estrl);
 	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-	but = bf_stock_button(_(" Delete "), G_CALLBACK(estrl_delete_clicked_lcb), estrl);
+	but = bf_generic_mnemonic_button(_(" Delete "), G_CALLBACK(estrl_delete_clicked_lcb), estrl);
 	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-	but = bf_stock_button(_(" Up "), G_CALLBACK(estrl_up_clicked_lcb), estrl);
+	but = bf_generic_mnemonic_button(_(" Up "), G_CALLBACK(estrl_up_clicked_lcb), estrl);
 	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-	but = bf_stock_button(_(" Down "), G_CALLBACK(estrl_down_clicked_lcb), estrl);
+	but = bf_generic_mnemonic_button(_(" Down "), G_CALLBACK(estrl_down_clicked_lcb), estrl);
 	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
-	but = bf_stock_button(_(" Sort "), G_CALLBACK(estrl_sort_clicked_lcb), estrl);
+	but = bf_generic_mnemonic_button(_(" Sort "), G_CALLBACK(estrl_sort_clicked_lcb), estrl);
 	gtk_box_pack_start(GTK_BOX(vbox2),but , TRUE, TRUE, 0);
 
 	hbox = gtk_hbutton_box_new();
@@ -396,7 +398,7 @@ void estrl_dialog(GList **which_list, gchar *title, gint what_list
 	  wx += optx + gtk_clist_optimal_column_width(GTK_CLIST(estrl->clist), 1);
 	  if( wx < gdk_screen_width() ) {
 	    gtk_clist_set_column_width(GTK_CLIST(estrl->clist), 0, optx );
-	    gtk_widget_set_usize(GTK_WIDGET(estrl->win), wx, 400);
+	    gtk_widget_set_size_request(GTK_WIDGET(estrl->win), wx, 400);
 	    gtk_widget_set_uposition(estrl->win, 
 				     (gdk_screen_width()-wx)/2,
 				     (gdk_screen_height() - 400)/2);
@@ -426,6 +428,14 @@ void debug_array(gchar **array) {
 }
 #endif
 
+/**
+ * count_array:
+ * @array: #gchar** with the NULL terminated array to count
+ *
+ * counts the number of entries in a NULL terminated array
+ *
+ * Return value: #gint with number of entries
+ */
 gint count_array(gchar **array) {
 	gint count=0;
 	gchar **tmpchar=array;
@@ -442,8 +452,19 @@ gint count_array(gchar **array) {
 	return count;
 }
 
-
-gchar *array_to_string(gchar **array, gchar delimiter) {
+/**
+ * array_to_string:
+ * @array: #gchar** with NULL terminated array
+ * @delimiter: #gchar with the delimiter character
+ *
+ * Converts a NULL terminated array to a string with a delimiter
+ * between the entries, and some characters backslash escaped
+ * like \t, \n, \\ and the delimiter. In Bluefish the : is the most 
+ * used delimiter
+ *
+ * Return value: #gchar* newly allocated string
+ */
+gchar *array_to_string(gchar **array) {
 	if (array) {
 		gchar **tmp, *escaped1, *finalstring;
 		gint newsize=1;
@@ -452,11 +473,11 @@ gchar *array_to_string(gchar **array, gchar delimiter) {
 		tmp = array;
 		while(*tmp) {
 			DEBUG_MSG("array_to_string, *tmp = %s\n", *tmp);
-			escaped1 = escapestring(*tmp, delimiter);
+			escaped1 = escape_string(*tmp, TRUE);
 			newsize += strlen(escaped1)+1;
 			finalstring = g_realloc(finalstring, newsize);
 			strcat(finalstring, escaped1);
-			finalstring[newsize-2] = delimiter;
+			finalstring[newsize-2] = ':';
 			finalstring[newsize-1] = '\0';
 			g_free(escaped1);
 			tmp++;
@@ -475,7 +496,18 @@ gchar *array_to_string(gchar **array, gchar delimiter) {
 
 #define ARRAYBLOCKSIZE 6
 #define BUFBLOCKSIZE 60
-gchar **string_to_array(gchar *string, gchar delimiter) {
+/**
+ * string_to_array:
+ * @string: #gchar* with the string to convert
+ * @delimiter: #gchar with the delimiter character
+ *
+ * breaks the string apart into a NULL terminated array
+ * using the delimiter character. \t, \\ and \n are also unescaped
+ * to their original characters
+ *
+ * Return value: #gchar** newly allocated NULL terminated array
+ */
+gchar **string_to_array(gchar *string) {
 	gchar **array;
 	gchar *tmpchar, *tmpchar2;
 	gchar *newstring;
@@ -494,8 +526,7 @@ gchar **string_to_array(gchar *string, gchar delimiter) {
 	while (*tmpchar != '\0') {
 		DEBUG_MSG("string_to_array, count=%d, newstring(%p)\n", count, newstring);
 		if (*tmpchar == '\\') {
-			tmpchar2 = tmpchar;
-			tmpchar2++;
+			tmpchar2 = tmpchar+1;
 			switch (*tmpchar2) {
 			case '\0':
 				newstring[count] = '\\';
@@ -508,17 +539,20 @@ gchar **string_to_array(gchar *string, gchar delimiter) {
 				newstring[count] = '\n';
 				tmpchar++;
 			break;
+			case 't':
+				newstring[count] = '\t';
+				tmpchar++;
+			break;
+			case ':':
+				newstring[count] = ':';
+				tmpchar++;
+			break;
 			default:
-				if (*tmpchar2 == delimiter) {
-					newstring[count] = delimiter;
-					tmpchar++;
-				} else {
-					DEBUG_MSG("string_to_array, weird, an unescaped backslash ?\n");
-					newstring[count] = '\\';
-				}
+				DEBUG_MSG("string_to_array, weird, an unescaped backslash ?\n");
+				newstring[count] = '\\';
 			break;
 			}
-		} else if (*tmpchar == delimiter) {
+		} else if (*tmpchar == ':') {
 			newstring[count] = '\0';  /* end of the current newstring */
 			DEBUG_MSG("string_to_array, newstring(%p)=%s\n", newstring, newstring);
 			array[arraycount] = g_strdup(newstring);
@@ -557,6 +591,67 @@ gchar **string_to_array(gchar *string, gchar delimiter) {
 	return array;
 }		
 
+/**
+ * array_from_arglist:
+ * @...: #gchar* with the first string, NULL terminated
+ *
+ * builds a NULL terminated array from the argumentlist to this function
+ *
+ * Return value: #gchar** with the array
+ */
+gchar **array_from_arglist(const gchar *string1, ...) {
+	gint numargs=1;
+	va_list args;
+	gchar *s;
+	gchar **retval, **index;
+
+	va_start (args, string1);
+	s = va_arg (args, gchar*);
+	while (s) {
+		numargs++;
+		s = va_arg (args, gchar*);
+	}
+	va_end (args);
+	DEBUG_MSG("array_from_arglist, numargs=%d\n", numargs);
+
+	index = retval = g_new(gchar *, numargs + 1);
+	*index = g_strdup(string1);
+
+	va_start (args, string1);
+	s = va_arg (args, gchar*);
+	while (s) {
+		index++;
+		*index = g_strdup(s);
+		s = va_arg (args, gchar*);
+	}
+	va_end (args);
+	index++;
+	*index = NULL;
+	return retval;
+}
+/**
+ * array_from_arglist:
+ * @allocate_strings: #gboolean if the strings should be newly allocated
+ * @...: #gchar* with the first string, NULL terminated
+ *
+ * builds a GList with strings (a stringlist), copied by reference or by content
+ *
+ * Return value: GList *
+ */
+GList *list_from_arglist(gboolean allocate_strings, ...) {
+	GList *retval=NULL;
+	va_list args;
+	gchar *s;
+	va_start(args, allocate_strings);
+	s = va_arg(args, gchar*);
+	while (s) {
+		retval = g_list_append(retval, s);
+		s = va_arg (args, gchar*);
+	}
+	va_end (args);
+	return retval;
+}
+
 GList *duplicate_stringlist(GList *list, gint dup_data) {
 	GList *retlist=NULL;
 	if (list) {
@@ -657,25 +752,25 @@ GList *duplicate_arraylist(GList *arraylist) {
  * gets a stringlist from a file
  */
 GList *get_list(const gchar * filename, GList * which_list, gboolean is_arraylist) {
-	char *tempstr, *tmpbuf;
-	FILE *fd;
-
 	DEBUG_MSG("get_stringlist, started with filename=%s\n", filename);
 	if (filename == NULL) {
 		return NULL;
 	}
-	tmpbuf = g_malloc(STRING_MAX_SIZE);
 	if (file_exists_and_readable(filename)) {
+		FILE *fd;
+		gchar *tmpbuf;
 		DEBUG_MSG("get_stringlist, opening %s\n", filename);
 		fd = fopen(filename, "r");
 		if (fd == NULL) {
 			return NULL;
 		}
+		tmpbuf = g_malloc(STRING_MAX_SIZE);
 		while (fgets(tmpbuf, STRING_MAX_SIZE, fd) != NULL) {
+			gchar *tempstr;
 			tmpbuf = trunc_on_char(tmpbuf, '\n');
 			tempstr = g_strdup(tmpbuf);
 			if (is_arraylist) {
-				gchar **temparr = string_to_array(tempstr, ':');
+				gchar **temparr = string_to_array(tempstr);
 				which_list = g_list_append(which_list, temparr);
 				g_free(tempstr);
 			} else {
@@ -684,8 +779,8 @@ GList *get_list(const gchar * filename, GList * which_list, gboolean is_arraylis
 			}
 		}
 		fclose(fd);
+		g_free(tmpbuf);
 	}
-	g_free(tmpbuf);
 	return which_list;
 }
 #ifdef __GNUC__
@@ -695,10 +790,17 @@ GList *get_stringlist(const gchar * filename, GList * which_list) {
 	return get_list(filename,which_list,FALSE);
 }
 
-/*****************************************************************************
- * stores a stringlist into a file
+/**
+ * put_stringlist_limited:
+ * @filename: #gchar* with the filename to store the list
+ * @which_list: #GList* with the list to store
+ * @maxentries: #gint only the LAST maxentries of the list will be stored
+ *
+ * stores the LAST maxentries entries of list which_list in file filename
+ * if maxentries is 0 all entries will be stored
+ *
+ * Return value: #gboolean TRUE on success, FALSE on failure
  */
-
 gboolean put_stringlist_limited(gchar * filename, GList * which_list, gint maxentries) {
 	FILE *fd;
 	GList *tmplist;
@@ -754,26 +856,88 @@ GList *remove_from_stringlist(GList *which_list, const gchar * string) {
 	return which_list;
 }
 
-GList *add_to_history_stringlist(GList *which_list, const gchar *string) {
+static void unlink_before(GList *tmplist) {
+	GList *prev = tmplist->prev;
+	if (prev) {
+		prev->next = NULL;
+	}
+	tmplist->prev = NULL;
+}
+
+GList *limit_stringlist(GList *which_list, gint num_entries, gboolean keep_end) {
+	GList *retlist, *freelist;
+	if (keep_end) {
+		gint len;
+		freelist = g_list_first(which_list);
+		len = g_list_length(freelist);
+		if (len <= num_entries) return which_list;
+		retlist = g_list_nth(freelist, len - num_entries);
+		unlink_before(retlist);
+	} else {
+		retlist = g_list_first(which_list);
+		freelist = g_list_nth(retlist, num_entries);
+		if (freelist) unlink_before(freelist);
+	}
+	if (freelist) free_stringlist(freelist);
+	return retlist;
+}
+
+/**
+ * add_to_history_stringlist:
+ * @which_list: #GList* the list to add to
+ * @string: #const gchar* with the string to add
+ * @recent_on_top: #gboolean, TRUE if the most recent entry is the one on top
+ * @move_if_exists: #gboolean, if TRUE do move existing entries to the most recent part
+ *
+ * adds string to the stringlist which_list
+ *
+ * if string exists in this list already and move_if_exists is TRUE, 
+ * it will be moved to the most recent part of the list (which is the 
+ * end or the beginning based on the value of recent_on_top
+ *
+ * Return value: GList* with the modified list
+ */
+GList *add_to_history_stringlist(GList *which_list, const gchar *string, gboolean recent_on_top, gboolean move_if_exists) {
 	if (string && strlen(string) ) {
 		GList *tmplist = g_list_first(which_list);
 		while (tmplist) {
 			if (strcmp((gchar *) tmplist->data, string) == 0) {
 				/* move this entry to the end */
-				DEBUG_MSG("add_to_history_stringlist, entry exists, moving to the end\n");
-				which_list = g_list_remove_link(which_list, tmplist);
-				return g_list_concat(which_list, tmplist);
+				if (move_if_exists) {
+					DEBUG_MSG("add_to_history_stringlist, entry %s exists, moving!\n", string);
+					which_list = g_list_remove_link(which_list, tmplist);
+					if (recent_on_top) {
+						return g_list_concat(tmplist, which_list);
+					} else {
+						return g_list_concat(which_list, tmplist);
+					}
+				} else {
+					return which_list;
+				}
 			}
 			tmplist = g_list_next(tmplist);
 		}
 		/* if we arrive here the string was not yet in the list */
-		DEBUG_MSG("add_to_history_stringlist, appending new entry\n");
-		which_list = g_list_append(which_list, g_strdup(string));
+		DEBUG_MSG("add_to_history_stringlist, adding new entry %s\n",string);
+		if (recent_on_top) {
+			which_list = g_list_prepend(which_list, g_strdup(string));
+		} else {
+			which_list = g_list_append(which_list, g_strdup(string));
+		}
 	}
 	return which_list;
 }
 
-/* designed for adding strings to colorlist, urllist, fontlist and targetlist */
+/**
+ * add_to_stringlist:
+ * @which_list: a #GList * to add to
+ * @string: a #const gchar * item you want to add to the list
+ * 
+ * this function will check if a string with same content exists already
+ * and if not it will add it to the list, it returns the new list pointer
+ * 
+ * Return value: the new GList *
+ **/
 GList *add_to_stringlist(GList * which_list, const gchar * string) {
 	if (string && strlen(string) ) {
 		GList *tmplist = g_list_first(which_list);
@@ -789,7 +953,17 @@ GList *add_to_stringlist(GList * which_list, const gchar * string) {
 	}
 	return which_list;
 }
-
+/**
+ * stringlist_to_string:
+ * @stringlist: a #GList * to convert
+ * @delimiter: a #const gchar * item with the delimiter
+ * 
+ * this function will convert a stringlist (GList that contains 
+ * only \0 terminated gchar* elements) to a string, putting the 
+ * delimiter inbetween all elements;
+ * 
+ * Return value: the gchar *
+ **/
 gchar *stringlist_to_string(GList *stringlist, gchar *delimiter) {
 	gchar *string, *tmp;
 	GList *tmplist;
@@ -804,3 +978,183 @@ gchar *stringlist_to_string(GList *stringlist, gchar *delimiter) {
 	}
 	return string;
 }
+/**
+ * array_n_strings_identical:
+ * @array1: #gchar**
+ * @array2: #gchar**
+ * @case_sensitive: #gboolean 
+ * @testlevel: #gint
+ *
+ * tests the first testlevel strings in the arrays if they are identical
+ * returns the first strcmp() value that is not 0, or 0 if all
+ * strings up to testlevel are identical.
+ *
+ * if BOTH array end before testlevel is reached, 0 is returned
+ * if ONE array ends before the other, -1 or 1 is returned
+ *
+ * Return value: #gint
+ */
+gint array_n_strings_identical(gchar **array1, gchar **array2, gboolean case_sensitive, gint testlevel) {
+	gint i=0, res=0;
+	while (i<testlevel && res==0) {
+		/*  array1[i]==array2[i] will only happen when they are both NULL	*/
+		if (array1[i] == NULL || array2[i] == NULL) {
+			/* NULL - NULL = 0
+			 * NULL - ptr = negative
+			 * ptr - NULL = positive */
+			return (gint)(array1[i] - array2[i]);
+		} else if (case_sensitive) {
+			res = strcmp(array1[i],array2[i]);
+		} else /*if (!case_sensitive)*/ {
+			res = strcasecmp(array1[i],array2[i]);
+		}
+		if (res !=0) return res;
+		i++;
+	}
+	return res;
+}
+/**
+ * arraylist_delete_identical:
+ * @thelist: #GList*
+ * @compare: #gchar**
+ * @testlevel: #gint
+ * @case_sensitive: #gboolean
+ *
+ * Deletes all arrays from 'arraylist' that are, up to testlevel, identical to array 'compare'
+ *
+ * Return value: the new list
+ */
+GList *arraylist_delete_identical(GList *thelist, gchar **compare, gint testlevel, gboolean case_sensitive) {
+	GList *tmplist = g_list_first(thelist);
+	while (tmplist) {
+		GList *nextlist = g_list_next(tmplist);
+		gchar **tmparr = tmplist->data;
+		if (array_n_strings_identical(compare, tmparr, case_sensitive, testlevel)==0) {
+			DEBUG_MSG("arraylist_delete_identical, %s and %s are identical, will delete %p from list\n",tmparr[0],compare[0], tmplist);
+			thelist = g_list_delete_link(thelist, tmplist);
+			DEBUG_MSG("arraylist_delete_identical, free array %p (%s)\n",tmparr,tmparr[0]);
+			g_strfreev(tmparr);
+		}
+		tmplist = nextlist;
+	}
+	return thelist;
+}
+
+/**
+ * arraylist_append_identical_from_list:
+ * @thelist: #GList*
+ * @source: #GList*
+ * @compare: #gchar**
+ * @testlevel: #gint
+ * @case_sensitive: #gboolean
+ *
+ * compares every array in 'source' with 'compare', and if it is identical up to 'testlevel', it will
+ * add the array to 'thelist'
+ *
+ * Return value: the new arraylist
+ */
+GList *arraylist_append_identical_from_list(GList *thelist, GList *source, gchar **compare, gint testlevel, gboolean case_sensitive) {
+	GList *tmplist = g_list_first(source);
+	while (tmplist) {
+		gchar **tmparr = tmplist->data;
+		if (array_n_strings_identical(compare, tmparr, case_sensitive, testlevel)==0) {
+			thelist = g_list_append(thelist, duplicate_stringarray(tmparr));
+		}
+		tmplist = g_list_next(tmplist);
+	}
+	return thelist;
+}
+
+/**
+ * arraylist_append_identical_from_file:
+ * @thelist: #GList*
+ * @sourcefilename: #const gchar*
+ * @compare: #gchar**
+ * @testlevel: #gint
+ * @case_sensitive: #gboolean
+ *
+ * compares every array read from 'sourcefilename' with 'compare', and if it is identical up to 'testlevel', it will
+ * add the array to 'thelist'
+ *
+ * Return value: the new arraylist
+ */
+GList *arraylist_append_identical_from_file(GList *thelist, const gchar *sourcefilename, gchar **compare, gint testlevel, gboolean case_sensitive) {
+	GList *sourcelist = get_list(sourcefilename,NULL,TRUE);
+	thelist = arraylist_append_identical_from_list(thelist, sourcelist, compare, testlevel, case_sensitive);
+	free_arraylist(sourcelist);
+	return thelist;
+}
+
+/**
+ * arraylist_value_exists:
+ * @arraylist: #GList*
+ * @value: #gchar**
+ * @testlevel: #gint
+ * @case_sensitive: #gboolean whether the test should be case sensitive
+ *
+ * tests for the occurence of an array with identical content as value
+ * in arraylist. It will only test the first testlevel strings in the array
+ * so if you want the test to check for a 100% identical array that number 
+ * should be high (9999 or so)
+ *
+ * Return value: #gboolean 
+ */
+gboolean arraylist_value_exists(GList *arraylist, gchar **value, gint testlevel, gboolean case_sensitive) {
+	GList *tmplist = g_list_first(arraylist);
+	while (tmplist) {
+		gchar **tmparr = tmplist->data;
+		if (array_n_strings_identical(value, tmparr, case_sensitive, testlevel)==0) {
+			return TRUE;
+		}
+		tmplist = g_list_next(tmplist);
+	}
+	return FALSE;
+}
+/**
+ * arraylist_load_new_identifiers_from_list:
+ * @mylist: #GList*
+ * @fromlist: #GList*
+ * @uniquelevel: #gint
+ *
+ * compares every entry in fromlist 
+ * with all entries in list mylist. Comparision is done uniquelevel deep
+ * by function arraylist_value_exists()
+ * those arrays that do _not_ match are appended to mylist which is returned
+ * at the end of the function
+ *
+ * Return value: #GList*
+ */
+GList *arraylist_load_new_identifiers_from_list(GList *mylist, GList *deflist, gint uniquelevel) {
+	GList *tmplist = g_list_first(deflist);
+	while (tmplist) {
+		gchar **tmparr = tmplist->data;
+		if (count_array(tmparr) >= uniquelevel) {
+			if (!arraylist_value_exists(mylist, tmparr, uniquelevel, TRUE)) {
+				DEBUG_MSG("arraylist_load_new_identifiers, adding %s to thelist\n",tmparr[0]);
+				mylist = g_list_append(mylist, duplicate_stringarray(tmparr));
+			}
+		}
+		tmplist = g_list_next(tmplist);
+	}
+	return mylist;
+}
+/**
+ * arraylist_load_new_identifiers_from_file:
+ * @mylist: #GList*
+ * @fromfilename: #gchar*
+ * @uniquelevel: #gint
+ *
+ * loads an arraylist from fromfilename and compares every entry 
+ * with all entries in list mylist. Comparision is done uniquelevel deep
+ * by function arraylist_value_exists()
+ * those arrays that do _not_ match are appended to mylist which is returned
+ * at the end of the function
+ *
+ * Return value: #GList*
+ */
+GList *arraylist_load_new_identifiers_from_file(GList *mylist, const gchar *fromfilename, gint uniquelevel) {
+	GList *deflist = get_list(fromfilename,NULL,TRUE);
+	mylist = arraylist_load_new_identifiers_from_list(mylist, deflist, uniquelevel);
+	free_arraylist(deflist);
+	return mylist;	
+}
